#!/bin/bash

if [ "$1" = "" ] || [ "$2" = "" ] || [ "$3" = "" ] || [ "$4" = "" ] || [ ! "$5" = "" ]; then
    echo "usage : $0 nom_groupe nom_du_module network adresse_br0"
    echo $@
    exit 1
fi
. /etc/eole/network_containers.conf
NAME=$1
CONTAINER=$2
NET=$3
BR0=$4
ROOTLXC=/opt/lxc
ROOT=${ROOTLXC}/default
PKG=$ROOTLXC/eole23.packages.tar
ROOTFS=$ROOTLXC/$NAME/rootfs
REP_DEF_FSTAB='/usr/share/eole/lxc/fstab'
FSTAB_DIR=$ROOTLXC/$NAME/fstab.d
FSTAB_FILE="$ROOTLXC/$NAME/fstab"
LOG="/var/log/isolation.log"
TMPL="/usr/share/eole/creole/distrib"
rm -f $LOG
function execute() {
    $1 >> $LOG 2>&1
    if [ ! $? = 0 ]; then
        echo "Erreur à la création du conteneur."
        echo "Pour plus d'information, lire les logs :"
        echo "$LOG"
        exit 1
    fi
}

#mount cgroup
if [ ! -d /cgroup ]; then
    rm -rf /cgroup
    mkdir /cgroup
fi
grep -q "none /cgroup cgroup" /etc/fstab
[ $? = 1 ] && echo "none /cgroup cgroup defaults 0 0" >> /etc/fstab

mount|grep -q cgroup
[ $? = 1 ] && mount /cgroup

if [ ! -d $ROOTLXC ]; then
    rm -rf $ROOTLXC
    if [ ! -L /var/lib/lxc ]; then
        mv /var/lib/lxc $ROOTLXC
        ln -s $ROOTLXC /var/lib/lxc
    fi
fi
#configuration de fstab
mkdir -p $FSTAB_DIR
if [ -e $REP_DEF_FSTAB/DEFAULT ]; then
   cp -f $REP_DEF_FSTAB/DEFAULT $FSTAB_DIR/
fi
if [ -e $REP_DEF_FSTAB/$CONTAINER ]; then
    cp -f $REP_DEF_FSTAB/$CONTAINER $FSTAB_DIR/
fi
cat $FSTAB_DIR/* > $FSTAB_FILE

if [ -e $FSTAB_FILE ]; then
    sed -i "s@{{ROOTFS}}@$ROOTFS@g" $FSTAB_FILE
    awk '{ print $2 }' < $FSTAB_FILE | while read rep
    do
        [ ! -d $rep ] && mkdir -p $rep
    done
else
    echo "Pas de fichier fstab pour $NAME"
fi

# Test si le contenaire existe
[ -d $ROOTFS/etc/ ] && echo "Le conteneur $NAME existe déjà" && exit 0

#Fix pour avoir un bon mappage de groupes et des users dans les containers par rapport au maitre
#attention le user eole n'est pas encore cree
mkdir $ROOTFS/etc/
cp /etc/group $ROOTFS/etc/
cp /etc/passwd $ROOTFS/etc/
awk 'BEGIN { sep=":"; FS=":" } {print $1 sep "*" sep $3 sep $4 sep $5 sep $6 sep $7 sep $8 sep $9; }' /etc/shadow > $ROOTFS/etc/shadow

if [ ! -e $PKG  ]; then
    DEV=${ROOT}/dev
    echo "Création de l'image du conteneur (veuillez patienter)"
    if [ -d $ROOT ]; then
        echo "$ROOT existe déjà"
        exit 1
    fi
    grep -q "^deb cdrom" /etc/apt/sources.list
    if [ $? = 0 ];then
	mount /dev/cdrom /cdrom
    	if [ ! $? = 0 ];then
		echo "Impossible de monter le cdrom"
		exit 1
    	fi
	echo "deb file:///cdrom/ubuntu lucid main multiverse universe local" > /etc/apt/sources.list
	source="file:///cdrom/ubuntu/"
	fromcd=1
    else
	source="http://archive.ubuntu.com/ubuntu/"
	fromcd=0
    fi
    execute "debootstrap --variant=minbase --arch i386 lucid $ROOT $source"
    #execute "debootstrap --variant=minbase --arch i386 lucid $ROOT http://archive.ubuntu.com/ubuntu/"
    if [ ! -d $DEV ]; then
        echo "Ne trouve pas $DEV"
        exit 1
    fi
    echo "Configuration de l'image du conteneur"
    rm -rf ${DEV}
    mkdir ${DEV}
    mknod -m 666 ${DEV}/null c 1 3
    mknod -m 666 ${DEV}/zero c 1 5
    mknod -m 666 ${DEV}/random c 1 8
    mknod -m 666 ${DEV}/urandom c 1 9
    mkdir -m 755 ${DEV}/pts
    mkdir -m 1777 ${DEV}/shm
    mknod -m 666 ${DEV}/tty c 5 0
    mknod -m 666 ${DEV}/tty0 c 4 0
    mknod -m 666 ${DEV}/tty1 c 4 1
    mknod -m 600 ${DEV}/console c 5 1
    mknod -m 666 ${DEV}/full c 1 7
    mknod -m 600 ${DEV}/initctl p
    mknod -m 666 ${DEV}/ptmx c 5 2

    echo "Creation de la clef SSH"
    [ ! -e /root/.ssh/id_rsa ] && ssh-keygen -t rsa -P '' -f /root/.ssh/id_rsa
    mkdir -p $ROOT/root/.ssh/
    cat /root/.ssh/id_rsa.pub > $ROOT/root/.ssh/authorized_keys
    chmod 700 $ROOT/root/.ssh/authorized_keys

    echo "Installation des paquets supplémentaires"
    if [ $fromcd = 1 ];then
	    . /etc/lsb-release
	    echo "deb file:///cdrom/ubuntu $DISTRIB_CODENAME main multiverse universe local" > $ROOT/etc/apt/sources.list
	    mkdir $ROOT/cdrom
	    mount --bind /cdrom $ROOT/cdrom
    else
    	cp -f /etc/apt/sources.list $ROOT/etc/apt/
    fi
    cp -f /usr/bin/apt-eole $ROOT/usr/bin/
    execute "chroot $ROOT apt-get update"
    execute "chroot $ROOT apt-eole install language-pack-fr gpgv openssh-server ubuntu-keyring eolebase-conteneur --force-yes"
    #Mise a jour de l'image qui servira pour les conteneurs
    execute "chroot $ROOT apt-eole -y --force-yes upgrade"
    execute "chroot $ROOT dpkg-divert --rename --divert /bin/mknod.real /bin/mknod"

    if [ $fromcd = 1 ];then
	    umount $ROOT/cdrom
    fi

    echo -e '#!/bin/bash\nmknod.real $@\nexit 0' > $ROOT/bin/mknod
    chmod +x $ROOT/bin/mknod

    echo "Configuration des locales"
    cp -f /etc/timezone $ROOT/etc/
    cp -f /etc/localtime $ROOT/etc/
    execute "chroot $ROOT update-locale LANG=\"fr_FR.UTF-8\" LANGUAGE=\"fr_FR.UTF-8\" LC_ALL=\"fr_FR.UTF-8\""

    echo "Configuration du démarrage"
    rm -f $ROOT/etc/init/tty{1,2,3,4,5,6}.conf
    mkdir -p $ROOT/var/run/network
    touch $ROOT/var/run/network/ifstate
    chroot $ROOT /usr/sbin/update-rc.d -f ondemand remove
    sed -i '/none            \/proc /d' $ROOT/lib/init/fstab
    sed -i '/none            \/sys /d' $ROOT/lib/init/fstab
    sed -i '/none            \/dev /d' $ROOT/lib/init/fstab
    sed -i 's@start on filesystem and net-device-up IFACE=lo@start on filesystem@g' $ROOT/etc/init/rc-sysinit.conf
    echo "# LXC – Fix init sequence to have LXC containers boot with upstart

# description “Fix LXC container - Lucid”

start on startup

task
pre-start script
mount -t proc proc /proc
mount -t devpts devpts /dev/pts
mount -t sysfs sys /sys
mount -t tmpfs varrun /var/run
mount -t tmpfs varlock /var/lock
mkdir -p /var/run/network
touch /var/run/utmp
chmod 664 /var/run/utmp
chown root.utmp /var/run/utmp
if [ \"\$(find /etc/network/ -name upstart -type f)\" ]; then
    chmod -x /etc/network/*/upstart || true
fi
end script

script
start networking
initctl emit filesystem --no-wait
initctl emit local-filesystems --no-wait
initctl emit virtual-filesystems --no-wait
init 2
end script" > $ROOT/etc/init/lxc.conf
    #On supprime tous les paquets du cache d'apt avant la compression"
    execute "chroot $ROOT apt-get clean"
    #correction du fichier /etc/rsyslog.conf dans le conteneur pour les messages noyau
    sed -i 's/$ModLoad imklog/#$ModLoad imklog/' $ROOT/etc/rsyslog.conf
    cd $ROOT
    echo "Compression de l'image du conteneur"
    execute "tar -cf $PKG *"
    cd - > /dev/null
    rm -rf $ROOT
fi
echo "Installation du conteneur $NAME (veuillez patienter)"

mkdir -p $ROOTFS
cd $ROOTFS
execute "tar -xf $PKG"
#On s'assure que le sources.list est bien le meme sur le conteneur que sur le maitre bug 1700
diff -Nru /etc/apt/sources.list $ROOTFS/etc/apt/sources.list 2>&1 > /dev/null
if ! [ $? = 0 ];then
    cp /etc/apt/sources.list $ROOTFS/etc/apt/sources.list
fi

echo "Configuration du conteneur $NAME"
echo "lxc.utsname = $NAME
lxc.tty = 2
lxc.network.type = veth
lxc.network.flags = up
lxc.network.link = br0
lxc.network.name = eth0
lxc.network.mtu = 1500
lxc.network.ipv4 = $NET
lxc.network.veth.pair = ve$NAME
lxc.rootfs = $ROOTFS
lxc.cgroup.devices.deny = a
# /dev/null and zero
lxc.cgroup.devices.allow = c 1:3 rwm
lxc.cgroup.devices.allow = c 1:5 rwm
# consoles
lxc.cgroup.devices.allow = c 5:1 rwm
lxc.cgroup.devices.allow = c 5:0 rwm
lxc.cgroup.devices.allow = c 4:0 rwm
lxc.cgroup.devices.allow = c 4:1 rwm
# /dev/{,u}random
lxc.cgroup.devices.allow = c 1:9 rwm
lxc.cgroup.devices.allow = c 1:8 rwm
# /dev/pts/* - pts namespaces are \"coming soon\"
lxc.cgroup.devices.allow = c 136:* rwm
lxc.cgroup.devices.allow = c 5:2 rwm
# rtc
lxc.cgroup.devices.allow = c 254:0 rwm
# partage du cache apt
lxc.mount = $FSTAB_FILE" > $ROOTLXC/$NAME/config.d/config

#changement du nom du module
IP="${NET%%/*}"
echo "127.0.0.1             localhost.localdomain localhost" > $ROOTLXC/$NAME/hosts.d/localhost

echo "$NAME" > $ROOTFS/etc/hostname

echo "auto lo
iface lo inet loopback

auto eth0
iface eth0 inet static
address $IP
netmask $adresse_netmask_br0
broadcast $adresse_broadcast_br0
network $adresse_network_br0
gateway $adresse_ip_br0" >  $ROOTFS/etc/network/interfaces

#Copie de la conf de ssh
mkdir -p $ROOTFS/etc/ssh/
cp -f /usr/share/eole/lxc/sshd_config $ROOTFS/etc/ssh/sshd_config
#echo "Démarrage du conteneur $NAME"
#lxc-create -f $ROOTLXC/$NAME/config -n $NAME
#execute "lxc-start -n $NAME -d"

#mettre le conteneur en demarrage automatique
echo 1 > $ROOTLXC/$NAME/on_boot

exit 0
